@@ -1,21 +1,26 @@
 use std::cmp::Ordering;
 use std::collections::{HashMap, HashSet};
 use std::str::FromStr;
+use std::sync::{Arc, Mutex};
 
 use anyhow::{anyhow, bail, Result};
 use bytesize::ByteSize;
 use chrono::{DateTime, Duration, Local};
 use clap::{AppSettings, Parser};
 use derive_more::Add;
-use futures::{future, TryStreamExt};
-use vlog::*;
+use itertools::Itertools;
+use log::*;
+use rayon::prelude::*;
 
 use super::{bytes, no_progress, progress_bytes, progress_counter, wait, warm_up, warm_up_command};
-use crate::backend::{Cache, DecryptFullBackend, DecryptReadBackend, FileType};
-use crate::blob::{BlobType, BlobTypeMap, NodeType, PackSizer, Repacker, TreeStreamerOnce};
+use crate::backend::{Cache, DecryptFullBackend, DecryptReadBackend, FileType, ReadBackend};
+use crate::blob::{
+    BlobType, BlobTypeMap, Initialize, NodeType, PackSizer, Repacker, Sum, TreeStreamerOnce,
+};
+use crate::commands::helpers::progress_spinner;
 use crate::id::Id;
 use crate::index::{IndexBackend, IndexCollector, IndexType, IndexedBackend, Indexer, ReadIndex};
-use crate::repo::{ConfigFile, IndexBlob, IndexFile, IndexPack, SnapshotFile};
+use crate::repo::{ConfigFile, HeaderEntry, IndexBlob, IndexFile, IndexPack, SnapshotFile};
 
 #[derive(Parser)]
 #[clap(global_setting(AppSettings::DeriveDisplayOrder))]
@@ -64,6 +69,10 @@ pub(super) struct Opts {
     #[clap(long, value_name = "TRUE/FALSE")]
     repack_cacheable_only: Option<bool>,
 
+    /// Do not repack packs which only needs to be resized
+    #[clap(long)]
+    no_resize: bool,
+
     /// Warm up needed data pack files by only requesting them without processing
     #[clap(long)]
     warm_up: bool,
@@ -77,7 +86,7 @@ pub(super) struct Opts {
     warm_up_wait: Option<humantime::Duration>,
 }
 
-pub(super) async fn execute(
+pub(super) fn execute(
     be: &(impl DecryptFullBackend + Unpin),
     cache: Option<Cache>,
     opts: Opts,
@@ -88,14 +97,13 @@ pub(super) async fn execute(
         bail!("--repack-uncompressed makes no sense for v1 repo!");
     }
 
-    v1!("reading index...");
     let mut index_files = Vec::new();
 
-    let p = progress_counter();
-    let mut stream = be.stream_all::<IndexFile>(p.clone()).await?;
+    let p = progress_counter("reading index...");
     let mut index_collector = IndexCollector::new(IndexType::OnlyTrees);
 
-    while let Some((id, index)) = stream.try_next().await? {
+    for index in be.stream_all::<IndexFile>(p.clone())? {
+        let (id, index) = index?;
         index_collector.extend(index.packs.clone());
         // we add the trees from packs_to_delete to the index such that searching for
         // used blobs doesn't abort if they are already marked for deletion
@@ -106,15 +114,14 @@ pub(super) async fn execute(
     p.finish();
 
     if let Some(cache) = &cache {
-        v1!("cleaning up packs from cache...");
-        cache
-            .remove_not_in_list(FileType::Pack, index_collector.tree_packs())
-            .await?;
+        let p = progress_spinner("cleaning up packs from cache...");
+        cache.remove_not_in_list(FileType::Pack, index_collector.tree_packs())?;
+        p.finish();
     }
     match (cache.is_some(), opts.cache_only) {
         (true, true) => return Ok(()),
         (false, true) => {
-            ve1!("Warning: option --cache-only used without a cache.");
+            warn!("Warning: option --cache-only used without a cache.");
             return Ok(());
         }
         _ => {}
@@ -122,19 +129,16 @@ pub(super) async fn execute(
 
     let (used_ids, total_size) = {
         let index = index_collector.into_index();
-        let total_size = BlobTypeMap::<()>::default().map(|tpe, _| index.total_size(&tpe));
+        let total_size = BlobTypeMap::init(|blob_type| index.total_size(&blob_type));
         let indexed_be = IndexBackend::new_from_index(&be.clone(), index);
-        let used_ids = find_used_blobs(&indexed_be, ignore_snaps).await?;
+        let used_ids = find_used_blobs(&indexed_be, ignore_snaps)?;
         (used_ids, total_size)
     };
 
     // list existing pack files
-    v1!("geting packs from repository...");
-    let existing_packs: HashMap<_, _> = be
-        .list_with_size(FileType::Pack)
-        .await?
-        .into_iter()
-        .collect();
+    let p = progress_spinner("geting packs from repository...");
+    let existing_packs: HashMap<_, _> = be.list_with_size(FileType::Pack)?.into_iter().collect();
+    p.finish();
 
     let mut pruner = Pruner::new(used_ids, existing_packs, index_files);
     pruner.count_used_blobs();
@@ -150,24 +154,29 @@ pub(super) async fn execute(
         opts.repack_uncompressed,
         &pack_sizer,
     )?;
-    pruner.decide_repack(&opts.max_repack, &opts.max_unused, opts.repack_uncompressed);
+    pruner.decide_repack(
+        &opts.max_repack,
+        &opts.max_unused,
+        opts.repack_uncompressed,
+        opts.no_resize,
+        &pack_sizer,
+    );
     pruner.check_existing_packs()?;
     pruner.filter_index_files(opts.instant_delete);
     pruner.print_stats();
 
     if opts.warm_up {
-        v1!("warming up needed data pack files...");
-        warm_up(be, pruner.repack_packs()).await?;
+        warm_up(be, pruner.repack_packs().into_iter())?;
     } else if opts.warm_up_command.is_some() {
-        v1!("warming up needed data pack files...");
         warm_up_command(
-            pruner.repack_packs(),
+            pruner.repack_packs().into_iter(),
             opts.warm_up_command.as_ref().unwrap(),
         )?;
     }
-    wait(opts.warm_up_wait).await;
+    wait(opts.warm_up_wait);
+
     if !opts.dry_run {
-        pruner.do_prune(be, opts, config).await?;
+        pruner.do_prune(be, opts, config)?;
     }
     Ok(())
 }
@@ -245,15 +254,6 @@ struct PruneStats {
     index_files: u64,
 }
 
-impl PruneStats {
-    fn total_blobs(&self) -> SizeStats {
-        self.blobs[BlobType::Tree] + self.blobs[BlobType::Data]
-    }
-    fn total_size(&self) -> SizeStats {
-        self.size[BlobType::Tree] + self.size[BlobType::Data]
-    }
-}
-
 #[derive(Debug)]
 struct PruneIndex {
     id: Id,
@@ -485,7 +485,7 @@ impl Pruner {
         // check that all used blobs are present in index
         for (id, count) in &self.used_ids {
             if *count == 0 {
-                eprintln!("used blob {} is missing", id);
+                error!("used blob {} is missing", id);
                 bail!("missing blobs");
             }
         }
@@ -591,6 +591,8 @@ impl Pruner {
         max_repack: &LimitOption,
         max_unused: &LimitOption,
         repack_uncompressed: bool,
+        no_resize: bool,
+        pack_sizer: &BlobTypeMap<PackSizer>,
     ) {
         let max_unused = match (repack_uncompressed, max_unused) {
             (true, _) => 0,
@@ -599,41 +601,48 @@ impl Pruner {
             // if percentag is given, we want to have
             // unused <= p/100 * size_after = p/100 * (size_used + unused)
             // which equals (1 - p/100) * unused <= p/100 * size_used
-            (false, LimitOption::Percentage(p)) => (p * self.stats.total_size().used) / (100 - p),
+            (false, LimitOption::Percentage(p)) => (p * self.stats.size.sum().used) / (100 - p),
         };
 
         let max_repack = match max_repack {
             LimitOption::Unlimited => u64::MAX,
             LimitOption::Size(size) => size.as_u64(),
-            LimitOption::Percentage(p) => (p * self.stats.total_size().total()),
+            LimitOption::Percentage(p) => p * self.stats.size.sum().total(),
         };
 
         self.repack_candidates.sort_unstable_by_key(|rc| rc.0);
         let mut resize_packs: BlobTypeMap<Vec<_>> = Default::default();
         let mut do_repack: BlobTypeMap<bool> = Default::default();
+        let mut repack_size: BlobTypeMap<u64> = Default::default();
 
         for (pi, repack_reason, index_num, pack_num) in std::mem::take(&mut self.repack_candidates)
         {
             let pack = &mut self.index_files[index_num].packs[pack_num];
+            let blob_type = pi.blob_type;
 
-            let repack_size_new =
-                self.stats.total_size().repack + (pi.unused_size + pi.used_size) as u64;
-            if repack_size_new >= max_repack
-                || (self.stats.total_size().unused_after_prune() < max_unused
+            let total_repack_size: u64 = repack_size.into_values().sum();
+            if total_repack_size + pi.used_size as u64 >= max_repack
+                || (self.stats.size.sum().unused_after_prune() < max_unused
                     && repack_reason == PartlyUsed
-                    && pi.blob_type == BlobType::Data)
+                    && blob_type == BlobType::Data)
+                || (repack_reason == SizeMismatch && no_resize)
             {
                 pack.set_todo(PackToDo::Keep, &pi, &mut self.stats);
             } else if repack_reason == SizeMismatch {
-                resize_packs[pack.blob_type].push((pi, index_num, pack_num));
+                resize_packs[blob_type].push((pi, index_num, pack_num));
+                repack_size[blob_type] += pi.used_size as u64;
             } else {
                 pack.set_todo(PackToDo::Repack, &pi, &mut self.stats);
-                do_repack[pack.blob_type] = true;
+                repack_size[blob_type] += pi.used_size as u64;
+                do_repack[blob_type] = true;
             }
         }
-        // resize_packs are only repacked if at least two packs of the blob_type are repacked
         for (blob_type, resize_packs) in resize_packs {
-            let todo = if do_repack[blob_type] || resize_packs.len() > 1 {
+            // packs in resize_packs are only repacked if we anyway repack this blob type or
+            // if the target pack size is reached for the blob type.
+            let todo = if do_repack[blob_type]
+                || repack_size[blob_type] > pack_sizer[blob_type].pack_size() as u64
+            {
                 PackToDo::Repack
             } else {
                 PackToDo::Keep
@@ -722,95 +731,91 @@ impl Pruner {
 
     fn print_stats(&self) {
         let pack_stat = &self.stats.packs;
-        let blob_stat = self.stats.total_blobs();
-        let size_stat = self.stats.total_size();
+        let blob_stat = self.stats.blobs.sum();
+        let size_stat = self.stats.size.sum();
 
-        v2!(
+        debug!(
             "used:   {:>10} blobs, {:>10}",
             blob_stat.used,
             bytes(size_stat.used)
         );
 
-        v2!(
+        debug!(
             "unused: {:>10} blobs, {:>10}",
             blob_stat.unused,
             bytes(size_stat.unused)
         );
-        v2!(
+        debug!(
             "total:  {:>10} blobs, {:>10}",
             blob_stat.total(),
             bytes(size_stat.total())
         );
 
-        v1!("");
-
-        v1!(
+        println!(
             "to repack: {:>10} packs, {:>10} blobs, {:>10}",
             pack_stat.repack,
             blob_stat.repack,
             bytes(size_stat.repack)
         );
-        v1!(
+        println!(
             "this removes:                {:>10} blobs, {:>10}",
             blob_stat.repackrm,
             bytes(size_stat.repackrm)
         );
-        v1!(
+        println!(
             "to delete: {:>10} packs, {:>10} blobs, {:>10}",
             pack_stat.unused,
             blob_stat.remove,
             bytes(size_stat.remove)
         );
         if !self.existing_packs.is_empty() {
-            v1!(
+            println!(
                 "unindexed: {:>10} packs,         ?? blobs, {:>10}",
                 self.existing_packs.len(),
                 bytes(self.stats.size_unref)
             );
         }
 
-        v1!(
+        println!(
             "total prune:                 {:>10} blobs, {:>10}",
             blob_stat.repackrm + blob_stat.remove,
             bytes(size_stat.repackrm + size_stat.remove + self.stats.size_unref)
         );
-        v1!(
+        println!(
             "remaining:                   {:>10} blobs, {:>10}",
             blob_stat.total_after_prune(),
             bytes(size_stat.total_after_prune())
         );
-        v1!(
+        println!(
             "unused size after prune: {:>10} ({:.2}% of remaining size)",
             bytes(size_stat.unused_after_prune()),
             size_stat.unused_after_prune() as f64 / size_stat.total_after_prune() as f64 * 100.0
         );
 
-        v1!("");
+        println!();
 
-        v1!(
+        println!(
             "packs marked for deletion: {:>10}, {:>10}",
             self.stats.packs_to_delete.total(),
             bytes(self.stats.size_to_delete.total()),
         );
-        v1!(
+        println!(
             " - complete deletion:      {:>10}, {:>10}",
             self.stats.packs_to_delete.remove,
             bytes(self.stats.size_to_delete.remove),
         );
-        v1!(
+        println!(
             " - keep marked:            {:>10}, {:>10}",
             self.stats.packs_to_delete.keep,
             bytes(self.stats.size_to_delete.keep),
         );
-        v1!(
+        println!(
             " - recover:                {:>10}, {:>10}",
             self.stats.packs_to_delete.recover,
             bytes(self.stats.size_to_delete.recover),
         );
 
-        v2!("");
-
-        v2!(
+        debug!(
             "index files to rebuild: {} / {}",
             self.index_files.len(),
             self.stats.index_files
@@ -826,12 +831,7 @@ impl Pruner {
             .collect()
     }
 
-    async fn do_prune(
-        mut self,
-        be: &impl DecryptFullBackend,
-        opts: Opts,
-        config: ConfigFile,
-    ) -> Result<()> {
+    fn do_prune(self, be: &impl DecryptFullBackend, opts: Opts, config: ConfigFile) -> Result<()> {
         let zstd = config.zstd()?;
         let mut be = be.clone();
         be.set_zstd(zstd);
@@ -847,39 +847,38 @@ impl Pruner {
         // - we don't know the number of packs generated by repacking
         // So, we simply use the current size of the blobs and an estimation of the pack
         // header size.
-        let tree_size_after_prune = self.stats.size[BlobType::Tree].total_after_prune()
-            + self.stats.blobs[BlobType::Tree].total_after_prune()
-                * IndexPack::HEADER_LEN_COMPRESSED as u64;
-        let data_size_after_prune = self.stats.size[BlobType::Data].total_after_prune()
-            + self.stats.blobs[BlobType::Data].total_after_prune()
-                * IndexPack::HEADER_LEN_COMPRESSED as u64;
 
-        let mut tree_repacker = Repacker::new(
+        let size_after_prune = BlobTypeMap::init(|blob_type| {
+            self.stats.size[blob_type].total_after_prune()
+                + self.stats.blobs[blob_type].total_after_prune()
+                    * HeaderEntry::ENTRY_LEN_COMPRESSED as u64
+        });
+
+        let tree_repacker = Repacker::new(
             be.clone(),
             BlobType::Tree,
             indexer.clone(),
             &config,
-            tree_size_after_prune,
+            size_after_prune[BlobType::Tree],
         )?;
 
-        let mut data_repacker = Repacker::new(
+        let data_repacker = Repacker::new(
             be.clone(),
             BlobType::Data,
             indexer.clone(),
             &config,
-            data_size_after_prune,
+            size_after_prune[BlobType::Data],
         )?;
 
         // mark unreferenced packs for deletion
         if !self.existing_packs.is_empty() {
             if opts.instant_delete {
-                v1!("removing not needed unindexed pack files...");
+                let p = progress_counter("removing unindexed packs...");
                 let existing_packs: Vec<_> =
                     self.existing_packs.into_iter().map(|(id, _)| id).collect();
-                be.delete_list(FileType::Pack, true, existing_packs, progress_counter())
-                    .await?;
+                be.delete_list(FileType::Pack, true, existing_packs, p)?;
             } else {
-                v1!("marking not needed unindexed pack files for deletion...");
+                info!("marking not needed unindexed pack files for deletion...");
                 for (id, size) in self.existing_packs {
                     let pack = IndexPack {
                         id,
@@ -887,7 +886,7 @@ impl Pruner {
                         time: Some(Local::now()),
                         blobs: Vec::new(),
                     };
-                    indexer.write().await.add_remove(pack).await?;
+                    indexer.write().unwrap().add_remove(pack)?;
                 }
             }
         }
@@ -895,118 +894,124 @@ impl Pruner {
         // process packs by index_file
         let p = match (self.index_files.is_empty(), self.stats.packs.repack > 0) {
             (true, _) => {
-                v1!("nothing to do!");
-                no_progress()
-            }
-            (false, true) => {
-                v1!("repacking packs and rebuilding index...");
-                progress_bytes()
-            }
-            (false, false) => {
-                v1!("rebuilding index...");
+                info!("nothing to do!");
                 no_progress()
             }
+            // TODO: Use a MultiProgressBar here
+            (false, true) => progress_bytes("repacking // rebuilding index..."),
+            (false, false) => progress_spinner("rebuilding index..."),
         };
 
-        p.set_length(self.stats.total_size().repack - self.stats.total_size().repackrm);
+        p.set_length(self.stats.size.sum().repack - self.stats.size.sum().repackrm);
 
         let mut indexes_remove = Vec::new();
-        let mut tree_packs_remove = Vec::new();
-        let mut data_packs_remove = Vec::new();
+        let tree_packs_remove = Arc::new(Mutex::new(Vec::new()));
+        let data_packs_remove = Arc::new(Mutex::new(Vec::new()));
 
-        let mut delete_pack = |pack: PrunePack| {
+        let delete_pack = |pack: PrunePack| {
             // delete pack
             match pack.blob_type {
-                BlobType::Data => data_packs_remove.push(pack.id),
-                BlobType::Tree => tree_packs_remove.push(pack.id),
+                BlobType::Data => data_packs_remove.lock().unwrap().push(pack.id),
+                BlobType::Tree => tree_packs_remove.lock().unwrap().push(pack.id),
             }
         };
 
-        for index in self.index_files {
-            for pack in index.packs.into_iter() {
-                match pack.to_do {
-                    PackToDo::Undecided => bail!("pack {} got no decicion what to do", pack.id),
-                    PackToDo::Keep => {
+        let used_ids = Arc::new(Mutex::new(self.used_ids));
+
+        let packs: Vec<_> = self
+            .index_files
+            .into_iter()
+            .map(|index| {
+                indexes_remove.push(index.id);
+                index
+            })
+            .flat_map(|index| index.packs)
+            .collect();
+
+        packs.into_par_iter().try_for_each(|pack| {
+            match pack.to_do {
+                PackToDo::Undecided => bail!("pack {} got no decicion what to do", pack.id),
+                PackToDo::Keep => {
+                    // keep pack: add to new index
+                    let pack = pack.into_index_pack();
+                    indexer.write().unwrap().add(pack)?;
+                }
+                PackToDo::Repack => {
+                    // TODO: repack in parallel
+                    for blob in &pack.blobs {
+                        if used_ids.lock().unwrap().remove(&blob.id).is_none() {
+                            // don't save duplicate blobs
+                            continue;
+                        }
+
+                        let repacker = match blob.tpe {
+                            BlobType::Data => &data_repacker,
+                            BlobType::Tree => &tree_repacker,
+                        };
+                        if opts.fast_repack {
+                            repacker.add_fast(&pack.id, blob)?;
+                        } else {
+                            repacker.add(&pack.id, blob)?;
+                        }
+                        p.inc(blob.length as u64);
+                    }
+                    if opts.instant_delete {
+                        delete_pack(pack);
+                    } else {
+                        // mark pack for removal
+                        let pack = pack.into_index_pack_with_time(self.time);
+                        indexer.write().unwrap().add_remove(pack)?;
+                    }
+                }
+                PackToDo::MarkDelete => {
+                    if opts.instant_delete {
+                        delete_pack(pack);
+                    } else {
+                        // mark pack for removal
+                        let pack = pack.into_index_pack_with_time(self.time);
+                        indexer.write().unwrap().add_remove(pack)?;
+                    }
+                }
+                PackToDo::KeepMarked => {
+                    if opts.instant_delete {
+                        delete_pack(pack);
+                    } else {
                         // keep pack: add to new index
                         let pack = pack.into_index_pack();
-                        indexer.write().await.add(pack).await?;
+                        indexer.write().unwrap().add_remove(pack)?;
                     }
-                    PackToDo::Repack => {
-                        // TODO: repack in parallel
-                        for blob in &pack.blobs {
-                            if self.used_ids.remove(&blob.id).is_none() {
-                                // don't save duplicate blobs
-                                continue;
-                            }
-
-                            let repacker = match blob.tpe {
-                                BlobType::Data => &mut data_repacker,
-                                BlobType::Tree => &mut tree_repacker,
-                            };
-                            if opts.fast_repack {
-                                repacker.add_fast(&pack.id, blob).await?;
-                            } else {
-                                repacker.add(&pack.id, blob).await?;
-                            }
-                            p.inc(blob.length as u64);
-                        }
-                        if opts.instant_delete {
-                            delete_pack(pack);
-                        } else {
-                            // mark pack for removal
-                            let pack = pack.into_index_pack_with_time(self.time);
-                            indexer.write().await.add_remove(pack).await?;
-                        }
-                    }
-                    PackToDo::MarkDelete => {
-                        if opts.instant_delete {
-                            delete_pack(pack);
-                        } else {
-                            // mark pack for removal
-                            let pack = pack.into_index_pack_with_time(self.time);
-                            indexer.write().await.add_remove(pack).await?;
-                        }
-                    }
-                    PackToDo::KeepMarked => {
-                        if opts.instant_delete {
-                            delete_pack(pack);
-                        } else {
-                            // keep pack: add to new index
-                            let pack = pack.into_index_pack();
-                            indexer.write().await.add_remove(pack).await?;
-                        }
-                    }
-                    PackToDo::Recover => {
-                        // recover pack: add to new index in section packs
-                        let pack = pack.into_index_pack_with_time(self.time);
-                        indexer.write().await.add(pack).await?;
-                    }
-                    PackToDo::Delete => delete_pack(pack),
                 }
+                PackToDo::Recover => {
+                    // recover pack: add to new index in section packs
+                    let pack = pack.into_index_pack_with_time(self.time);
+                    indexer.write().unwrap().add(pack)?;
+                }
+                PackToDo::Delete => delete_pack(pack),
             }
-            indexes_remove.push(index.id);
-        }
-        tree_repacker.finalize().await?;
-        data_repacker.finalize().await?;
-        indexer.write().await.finalize().await?;
+            Ok(())
+        })?;
+        tree_repacker.finalize()?;
+        data_repacker.finalize()?;
+        indexer.write().unwrap().finalize()?;
         p.finish();
 
+        // get variables out of Arc<Mutex<_>>
+        let data_packs_remove = std::mem::take::<Vec<_>>(&mut data_packs_remove.lock().unwrap());
+        let tree_packs_remove = std::mem::take::<Vec<_>>(&mut tree_packs_remove.lock().unwrap());
+
         if !data_packs_remove.is_empty() {
-            v1!("removing old data pack files...");
-            be.delete_list(FileType::Pack, false, data_packs_remove, progress_counter())
-                .await?;
+            let p = progress_counter("removing old data packs...");
+            be.delete_list(FileType::Pack, false, data_packs_remove, p)?;
         }
 
         if !tree_packs_remove.is_empty() {
-            v1!("removing old tree pack files...");
-            be.delete_list(FileType::Pack, true, tree_packs_remove, progress_counter())
-                .await?;
+            let p = progress_counter("removing old tree packs...");
+            be.delete_list(FileType::Pack, true, tree_packs_remove, p)?;
         }
 
         if !indexes_remove.is_empty() {
-            v1!("removing old index files...");
-            be.delete_list(FileType::Index, true, indexes_remove, progress_counter())
-                .await?;
+            let p = progress_counter("removing old index files...");
+            be.delete_list(FileType::Index, true, indexes_remove, p)?;
         }
 
         Ok(())
@@ -1083,32 +1088,32 @@ impl PackInfo {
 }
 
 // find used blobs in repo
-async fn find_used_blobs(
+fn find_used_blobs(
     index: &(impl IndexedBackend + Unpin),
     ignore_snaps: Vec<Id>,
 ) -> Result<HashMap<Id, u8>> {
     let ignore_snaps: HashSet<_> = ignore_snaps.into_iter().collect();
-    v1!("reading snapshots...");
 
-    let p = progress_counter();
+    let p = progress_counter("reading snapshots...");
+    let list = index
+        .be()
+        .list(FileType::Snapshot)?
+        .into_iter()
+        .filter(|id| !ignore_snaps.contains(id))
+        .collect();
     let snap_trees: Vec<_> = index
         .be()
-        .stream_all::<SnapshotFile>(p.clone())
-        .await?
-        // TODO: it would even better to give ignore_snaps to the streaming function instead
-        // if reading and then filtering the snapshot
-        .try_filter(|(id, _)| future::ready(!ignore_snaps.contains(id)))
+        .stream_list::<SnapshotFile>(list, p.clone())?
+        .into_iter()
         .map_ok(|(_, snap)| snap.tree)
-        .try_collect()
-        .await?;
+        .try_collect()?;
     p.finish();
 
-    v1!("finding used blobs...");
     let mut ids: HashMap<_, _> = snap_trees.iter().map(|id| (*id, 0)).collect();
+    let p = progress_counter("finding used blobs...");
 
-    let mut tree_streamer =
-        TreeStreamerOnce::new(index.clone(), snap_trees, progress_counter()).await?;
-    while let Some(item) = tree_streamer.try_next().await? {
+    let mut tree_streamer = TreeStreamerOnce::new(index.clone(), snap_trees, p)?;
+    while let Some(item) = tree_streamer.next().transpose()? {
         let (_, tree) = item;
         for node in tree.nodes() {
             match node.node_type() {
