@@ -9,11 +9,11 @@ pub(super) struct Opts {
     force: bool,
 }
 
-pub(super) async fn execute(opts: Opts) -> Result<()> {
+pub(super) fn execute(opts: Opts) -> Result<()> {
     let status = self_update::backends::github::Update::configure()
         .repo_owner("rustic-rs")
         .repo_name("rustic")
-        .bin_name("rustic-rs")
+        .bin_name("rustic")
         .show_download_progress(true)
         .current_version(cargo_crate_version!())
         .no_confirm(opts.force)
