@@ -40,7 +40,13 @@ mod id;
 mod index;
 mod repo;
 
-#[tokio::main]
-async fn main() -> Result<()> {
-    commands::execute().await
+fn main() -> Result<()> {
+    // this is a workaround until unix_sigpipe (https://github.com/rust-lang/rust/issues/97889) is available.
+    // See also https://github.com/rust-lang/rust/issues/46016
+    #[allow(unsafe_code)]
+    unsafe {
+        libc::signal(libc::SIGPIPE, libc::SIG_DFL);
+    }
+
+    commands::execute()
 }
